package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;

public class IDEAEngine implements BlockCipher {
   protected static final int BLOCK_SIZE = 8;
   private static final int MASK = 65535;
   private static final int BASE = 65537;
   private int[] workingKey = null;

   private int bytesToWord(byte[] in, int inOff) {
      return (in[inOff] << 8 & '\uff00') + (in[inOff + 1] & 255);
   }

   private void wordToBytes(int word, byte[] out, int outOff) {
      out[outOff] = (byte)(word >>> 8);
      int var4 = outOff + 1;
      out[var4] = (byte)word;
   }

   private int mul(int x, int y) {
      int var10000;
      if (x == 0) {
         var10000 = 65537 - y;
      } else if (y == 0) {
         var10000 = 65537 - x;
      } else {
         int var3;
         var10000 = (var3 = x * y) & '\uffff';
         x = var10000 - (var3 >>>= 16);
         byte var4;
         if (var10000 < var3) {
            var4 = 1;
         } else {
            var4 = 0;
         }

         var10000 = x + var4;
      }

      return var10000 & '\uffff';
   }

   private void ideaFunc(int[] workingKey, byte[] in, int inOff, byte[] out, int outOff) {
      int var6 = 0;
      int var7 = this.bytesToWord(in, inOff);
      int var8 = this.bytesToWord(in, inOff + 2);
      int var9 = this.bytesToWord(in, inOff + 4);
      int in1 = this.bytesToWord(in, inOff + 6);

      int var10000;
      for(inOff = 0; inOff < 8; var6 = var10000) {
         var7 = this.mul(var7, workingKey[var6++]);
         int var10002 = var8 = var8 + workingKey[var6++] & '\uffff';
         int var10003 = var9 = var9 + workingKey[var6++] & '\uffff';
         int var10006 = in1;
         in1 = var6 + 1;
         var6 = this.mul(var10006, workingKey[var6]);
         int var10 = var10003 ^ var7;
         int var11 = var10002 ^ var6;
         var10000 = var10 = this.mul(var10, workingKey[in1++]);
         var10003 = in1;
         var10 = var11 + var10 & '\uffff';
         ++in1;
         var10000 = var11 = var10000 + (var10 = this.mul(var10, workingKey[var10003])) & '\uffff';
         int var10001 = var8;
         var7 ^= var10;
         var6 ^= var11;
         var8 = var10 ^ var9;
         var9 = var10000 ^ var10001;
         ++inOff;
         var10000 = in1;
         in1 = var6;
      }

      int var12 = var6 + 1;
      this.wordToBytes(this.mul(var7, workingKey[var6]), out, outOff);
      this.wordToBytes(var9 + workingKey[var12++], out, outOff + 2);
      this.wordToBytes(var8 + workingKey[var12++], out, outOff + 4);
      this.wordToBytes(this.mul(in1, workingKey[var12]), out, outOff + 6);
   }

   private int[] expandKey(byte[] uKey) {
      int[] var2 = new int[52];
      int var3;
      if (uKey.length < 16) {
         byte[] var10000 = uKey;
         byte[] var10001 = uKey;
         byte[] var10002 = uKey;
         uKey = new byte[16];
         var3 = 16 - var10002.length;
         int var4 = var10001.length;
         System.arraycopy(var10000, 0, uKey, var3, var4);
      }

      for(var3 = 0; var3 < 8; ++var3) {
         var2[var3] = this.bytesToWord(uKey, var3 * 2);
      }

      for(int var5 = 8; var5 < 52; ++var5) {
         int uKey1;
         if ((uKey1 = var5 & 7) < 6) {
            var2[var5] = ((var2[var5 - 7] & 127) << 9 | var2[var5 - 6] >> 7) & '\uffff';
         } else if (uKey1 == 6) {
            var2[var5] = ((var2[var5 - 7] & 127) << 9 | var2[var5 - 14] >> 7) & '\uffff';
         } else {
            var2[var5] = ((var2[var5 - 15] & 127) << 9 | var2[var5 - 14] >> 7) & '\uffff';
         }
      }

      return var2;
   }

   private int mulInv(int x) {
      if (x < 2) {
         return x;
      } else {
         int var5 = 1;
         int var2 = 65537 / x;

         int var4;
         for(int var3 = 65537 % x; var3 != 1; x = var4) {
            int var10000 = x;
            x /= var3;
            var10000 = var4 = var10000 % var3;
            var5 = var5 + var2 * x & '\uffff';
            if (var10000 == 1) {
               return var5;
            }

            x = var3 / var4;
            var3 %= var4;
            var2 = var2 + var5 * x & '\uffff';
         }

         return 1 - var2 & '\uffff';
      }
   }

   private int[] invertKey(int[] inKey) {
      int[] var2;
      int[] var10000 = var2 = new int[52];
      int var3 = this.mulInv(inKey[0]);
      int var4 = this.addInv(inKey[1]);
      int var5 = this.addInv(inKey[2]);
      int var6 = 4;
      var10000[51] = this.mulInv(inKey[3]);
      var10000[50] = var5;
      var10000[49] = var4;
      var4 = 48;
      var10000[48] = var3;

      for(var3 = 1; var3 < 8; var6 = var5) {
         var5 = var6 + 1;
         var6 = inKey[var6];
         int var7 = inKey[var5++];
         var2[var4 += -1] = var7;
         var2[var4 += -1] = var6;
         var6 = this.mulInv(inKey[var5++]);
         var7 = this.addInv(inKey[var5++]);
         int var8 = this.addInv(inKey[var5++]);
         int var9 = this.mulInv(inKey[var5++]);
         var2[var4 += -1] = var9;
         var2[var4 += -1] = var7;
         var2[var4 += -1] = var8;
         var2[var4 += -1] = var6;
         ++var3;
      }

      int var10 = var6 + 1;
      int inKey1 = inKey[var6];
      var3 = inKey[var10++];
      var2[var4 += -1] = var3;
      var2[var3 = var4 + -1] = inKey1;
      inKey1 = this.mulInv(inKey[var10++]);
      var4 = this.addInv(inKey[var10++]);
      var5 = this.addInv(inKey[var10++]);
      var10 = this.mulInv(inKey[var10]);
      var2[var3 += -1] = var10;
      var2[var10 = var3 + -1] = var5;
      int var10002 = var10 + -1;
      var2[var10 + -1] = var4;
      var2[var10002 + -1] = inKey1;
      return var2;
   }

   private int[] generateWorkingKey(boolean forEncryption, byte[] userKey) {
      return forEncryption ? this.expandKey(userKey) : this.invertKey(this.expandKey(userKey));
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof KeyParameter) {
         this.workingKey = this.generateWorkingKey(forEncryption, ((KeyParameter)params).getKey());
      } else {
         throw new IllegalArgumentException("invalid parameter passed to IDEA init - " + params.getClass().getName());
      }
   }

   public String getAlgorithmName() {
      return "IDEA";
   }

   public int getBlockSize() {
      return 8;
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int[] var5;
      if ((var5 = this.workingKey) != null) {
         if (inOff + 8 <= in.length) {
            if (outOff + 8 <= out.length) {
               this.ideaFunc(var5, in, inOff, out, outOff);
               return 8;
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException("IDEA engine not initialised");
      }
   }

   public void reset() {
   }

   int addInv(int x) {
      return 0 - x & '\uffff';
   }
}
